function [AMcom, FMcom] = DESA1s(sig,wo)
%
% DES (Discrete Energy Separation) Algorithm. Returns the AM and FM
% components of the signal. If the signal is not a mono-component AM-FM
% signal then problems may occur, which are reported. The algorithm is
% described at "Energy Separation in Signal Modulations with Application
% to Speech Analysis" paper of Maragos et al. (1993).
%
%  ----INPUT PARAMETERS----
%    sig: input AM-FM signal
%    wo: center frequency of FM. 
%
%  ----OUTPUT PARAMENTERS----
%    AMcom: Amplitude Modulated component
%    FMcom: Frequency Modulated component
%
% (c) Yannis Pantazis, 2007, csd, uoc
%     mail: pantazis@csd.uoc.gr

slen = length(sig);
y = sig(2:slen) - sig(1:slen-1);   % difference

Yx = Teager(sig);    % apply Teager operator
Yy = Teager(y);
Yz = Teager(y(2:length(y)));

L = length(Yz);
Yx = Yx(2:L+1);
Yy = Yy(1:L);

% perform smoothing
fsm = [1 6 15 20 15 6 1];

Yx = filter(fsm,1,Yx);
Yy = filter(fsm,1,Yy);
Yz = filter(fsm,1,Yz);

% perform DES Algorithm
FMcom = zeros(1,L);
AMcom = zeros(1,L);

G = 1-(Yy+Yz)./(4*Yx);

% if a problem occurs, then, a default value is used.
in1 = find(abs(G)>=1);
G(in1) = cos(wo)*ones(length(in1),1);

FMcom = acos(G)';

in = find(Yx>=0);
AMcom(in) = sqrt(Yx(in)./(1-G(in).^2))';

% perform smoothing
FMcom = medfilt(FMcom,5);
AMcom = medfilt(AMcom,5);

fprintf(1, 'There are %d problematic samples out of %d\n', length(in1), slen);
